// ignore_for_file: public_member_api_docs

/// IdUid represents a compound of an ID, which is locally unique, i.e. inside
/// an entity, and a UID, which is globally unique, i.e. for the entire model.
///
/// It is serialized as two numerical values concatenated using a colon (`:`).
/// See the documentation for more information on
///  * [IDs](https://docs.objectbox.io/advanced/meta-model-ids-and-uids#ids)
///  * [UIDs](https://docs.objectbox.io/advanced/meta-model-ids-and-uids#uids)
class IdUid {
  /*late*/ int _id, _uid;

  int get id => _id;

  set id(int id) {
    if (id < 0 || id > ((1 << 63) - 1)) {
      throw Exception('id out of bounds: $id');
    }
    _id = id;
  }

  int get uid => _uid;

  set uid(int uid) {
    if (uid < 0 || uid > ((1 << 63) - 1)) {
      throw Exception('uid out of bounds: $uid');
    }
    _uid = uid;
  }

  IdUid(int newId, int newUid) {
    id = newId;
    uid = newUid;
  }

  IdUid.fromString(String /*?*/ str) {
    if (str == null || str == '' || str == '0:0') {
      _id = 0;
      _uid = 0;
      return;
    }

    var spl = str.split(':');
    if (spl.length != 2) {
      throw Exception(
          'IdUid has invalid format, wrong number of columns: $str');
    }
    id = int.parse(spl[0]);
    uid = int.parse(spl[1]);
  }

  IdUid.empty()
      : _id = 0,
        _uid = 0;

  bool get isEmpty => _id == 0 && _uid == 0;

  @override
  String toString() => '$_id:$_uid';
}
